@@ -7,30 +7,35 @@ from dask.array.utils import assert_eq
 import numpy as np
 
 from dask.array.core import Array
-from dask.array.gufunc import _parse_gufunc_signature, _validate_normalize_axes, apply_gufunc,gufunc, as_gufunc
+from dask.array.gufunc import (
+    _parse_gufunc_signature,
+    _validate_normalize_axes,
+    apply_gufunc,
+    gufunc,
+    as_gufunc,
+)
+from dask.array.utils import IS_NEP18_ACTIVE
 
 
 # Copied from `numpy.lib.test_test_function_base.py`:
 def test__parse_gufunc_signature():
-    assert_equal(_parse_gufunc_signature('(x)->()'), ([('x',)], ()))
-    assert_equal(_parse_gufunc_signature('(x,y)->()'),
-                 ([('x', 'y')], ()))
-    assert_equal(_parse_gufunc_signature('(x),(y)->()'),
-                 ([('x',), ('y',)], ()))
-    assert_equal(_parse_gufunc_signature('(x)->(y)'),
-                 ([('x',)], ('y',)))
-    assert_equal(_parse_gufunc_signature('(x)->(y),()'),
-                 ([('x',)], [('y',), ()]))
-    assert_equal(_parse_gufunc_signature('(),(a,b,c),(d)->(d,e)'),
-                 ([(), ('a', 'b', 'c'), ('d',)], ('d', 'e')))
+    assert_equal(_parse_gufunc_signature("(x)->()"), ([("x",)], ()))
+    assert_equal(_parse_gufunc_signature("(x,y)->()"), ([("x", "y")], ()))
+    assert_equal(_parse_gufunc_signature("(x),(y)->()"), ([("x",), ("y",)], ()))
+    assert_equal(_parse_gufunc_signature("(x)->(y)"), ([("x",)], ("y",)))
+    assert_equal(_parse_gufunc_signature("(x)->(y),()"), ([("x",)], [("y",), ()]))
+    assert_equal(
+        _parse_gufunc_signature("(),(a,b,c),(d)->(d,e)"),
+        ([(), ("a", "b", "c"), ("d",)], ("d", "e")),
+    )
     with pytest.raises(ValueError):
-        _parse_gufunc_signature('(x)(y)->()')
+        _parse_gufunc_signature("(x)(y)->()")
     with pytest.raises(ValueError):
-        _parse_gufunc_signature('(x),(y)->')
+        _parse_gufunc_signature("(x),(y)->")
     with pytest.raises(ValueError):
-        _parse_gufunc_signature('((x))->(x)')
+        _parse_gufunc_signature("((x))->(x)")
     with pytest.raises(ValueError):
-        _parse_gufunc_signature('(x)->(x),')
+        _parse_gufunc_signature("(x)->(x),")
 
 
 def test_apply_gufunc_axes_input_validation_01():
@@ -56,63 +61,63 @@ def test_apply_gufunc_axes_input_validation_01():
 
 def test__validate_normalize_axes_01():
     with pytest.raises(ValueError):
-        _validate_normalize_axes([(1, 0)], None, False, [('i', 'j')], ('j',))
+        _validate_normalize_axes([(1, 0)], None, False, [("i", "j")], ("j",))
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes([0, 0], None, False, [('i', 'j')], ('j',))
+        _validate_normalize_axes([0, 0], None, False, [("i", "j")], ("j",))
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes([(0,), 0], None, False, [('i', 'j')], ('j',))
+        _validate_normalize_axes([(0,), 0], None, False, [("i", "j")], ("j",))
 
-    i, o = _validate_normalize_axes([(1, 0), 0], None, False, [('i', 'j')], ('j',))
+    i, o = _validate_normalize_axes([(1, 0), 0], None, False, [("i", "j")], ("j",))
     assert i == [(1, 0)]
     assert o == [(0,)]
 
 
 def test__validate_normalize_axes_02():
-    i, o = _validate_normalize_axes(None, 0, False, [('i', ), ('i', )], ())
+    i, o = _validate_normalize_axes(None, 0, False, [("i",), ("i",)], ())
     assert i == [(0,), (0,)]
     assert o == [()]
 
-    i, o = _validate_normalize_axes(None, 0, False, [('i',)], ('i',))
+    i, o = _validate_normalize_axes(None, 0, False, [("i",)], ("i",))
     assert i == [(0,)]
     assert o == [(0,)]
 
-    i, o = _validate_normalize_axes(None, 0, True, [('i',), ('i',)], ())
+    i, o = _validate_normalize_axes(None, 0, True, [("i",), ("i",)], ())
     assert i == [(0,), (0,)]
     assert o == [(0,)]
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes(None, (0,), False, [('i',), ('i',)], ())
+        _validate_normalize_axes(None, (0,), False, [("i",), ("i",)], ())
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes(None, 0, False, [('i',), ('j',)], ())
+        _validate_normalize_axes(None, 0, False, [("i",), ("j",)], ())
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes(None, 0, False, [('i',), ('j',)], ('j',))
+        _validate_normalize_axes(None, 0, False, [("i",), ("j",)], ("j",))
 
 
 def test__validate_normalize_axes_03():
-    i, o = _validate_normalize_axes(None, 0, True, [('i',)], ())
+    i, o = _validate_normalize_axes(None, 0, True, [("i",)], ())
     assert i == [(0,)]
     assert o == [(0,)]
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes(None, 0, True, [('i',)], ('i',))
+        _validate_normalize_axes(None, 0, True, [("i",)], ("i",))
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes([(0, 1), (0, 1)], None, True, [('i', 'j')], ('i', 'j'))
+        _validate_normalize_axes([(0, 1), (0, 1)], None, True, [("i", "j")], ("i", "j"))
 
     with pytest.raises(ValueError):
-        _validate_normalize_axes([(0,), (0,)], None, True, [('i',), ('j',)], ())
+        _validate_normalize_axes([(0,), (0,)], None, True, [("i",), ("j",)], ())
 
 
 def test_apply_gufunc_01():
     def stats(x):
         return np.mean(x, axis=-1), np.std(x, axis=-1)
+
     a = da.random.normal(size=(10, 20, 30), chunks=(5, 5, 30))
-    mean, std = apply_gufunc(stats, "(i)->(),()", a,
-                             output_dtypes=2 * (a.dtype,))
+    mean, std = apply_gufunc(stats, "(i)->(),()", a, output_dtypes=2 * (a.dtype,))
     assert mean.compute().shape == (10, 20)
     assert std.compute().shape == (10, 20)
 
@@ -120,29 +125,34 @@ def test_apply_gufunc_01():
 def test_apply_gufunc_01b():
     def stats(x):
         return np.mean(x, axis=-1), np.std(x, axis=-1)
+
     a = da.random.normal(size=(10, 20, 30), chunks=5)
-    mean, std = apply_gufunc(stats, "(i)->(),()", a,
-                             output_dtypes=2 * (a.dtype,),
-                             allow_rechunk=True)
+    mean, std = apply_gufunc(
+        stats, "(i)->(),()", a, output_dtypes=2 * (a.dtype,), allow_rechunk=True
+    )
     assert mean.compute().shape == (10, 20)
     assert std.compute().shape == (10, 20)
 
 
-@pytest.mark.parametrize('vectorize', [False, True])
+@pytest.mark.parametrize("vectorize", [False, True])
 def test_apply_gufunc_output_dtypes_string(vectorize):
     def stats(x):
         return np.mean(x, axis=-1)
+
     a = da.random.normal(size=(10, 20, 30), chunks=(5, 5, 30))
     mean = apply_gufunc(stats, "(i)->()", a, output_dtypes="f", vectorize=vectorize)
     assert mean.compute().shape == (10, 20)
 
 
-@pytest.mark.parametrize('vectorize', [False, True])
+@pytest.mark.parametrize("vectorize", [False, True])
 def test_apply_gufunc_output_dtypes_string_many_outputs(vectorize):
     def stats(x):
         return np.mean(x, axis=-1), np.std(x, axis=-1)
+
     a = da.random.normal(size=(10, 20, 30), chunks=(5, 5, 30))
-    mean, std = apply_gufunc(stats, "(i)->(),()", a, output_dtypes=("f", "f"), vectorize=vectorize)
+    mean, std = apply_gufunc(
+        stats, "(i)->(),()", a, output_dtypes=("f", "f"), vectorize=vectorize
+    )
     assert mean.compute().shape == (10, 20)
     assert std.compute().shape == (10, 20)
 
@@ -151,15 +161,19 @@ def test_apply_gufunc_pass_additional_kwargs():
     def foo(x, bar):
         assert bar == 2
         return x
-    ret = apply_gufunc(foo, "()->()", 1., output_dtypes="f", bar=2)
-    assert_eq(ret, np.array(1., dtype="f"))
+
+    ret = apply_gufunc(foo, "()->()", 1.0, output_dtypes="f", bar=2)
+    assert_eq(ret, np.array(1.0, dtype="f"))
 
 
-@pytest.mark.xfail(reason="Currently np.einsum doesn't seem to broadcast correctly for this case")
+@pytest.mark.xfail(
+    reason="Currently np.einsum doesn't seem to broadcast correctly for this case"
+)
 def test_apply_gufunc_02():
     def outer_product(x, y):
         return np.einsum("...i,...j->...ij", x, y)
-    a = da.random.normal(size=(   20, 30), chunks=(5, 30))
+
+    a = da.random.normal(size=(20, 30), chunks=(5, 30))
     b = da.random.normal(size=(10, 1, 40), chunks=(10, 1, 40))
     c = apply_gufunc(outer_product, "(i),(j)->(i,j)", a, b, output_dtypes=a.dtype)
     assert c.compute().shape == (10, 20, 30, 40)
@@ -168,6 +182,7 @@ def test_apply_gufunc_02():
 def test_apply_gufunc_scalar_output():
     def foo():
         return 1
+
     x = apply_gufunc(foo, "->()", output_dtypes=int)
     assert x.compute() == 1
 
@@ -175,8 +190,9 @@ def test_apply_gufunc_scalar_output():
 def test_apply_gufunc_elemwise_01():
     def add(x, y):
         return x + y
-    a = da.from_array(np.array([1, 2, 3]), chunks=2, name='a')
-    b = da.from_array(np.array([1, 2, 3]), chunks=2, name='b')
+
+    a = da.from_array(np.array([1, 2, 3]), chunks=2, name="a")
+    b = da.from_array(np.array([1, 2, 3]), chunks=2, name="b")
     z = apply_gufunc(add, "(),()->()", a, b, output_dtypes=a.dtype)
     assert_eq(z, np.array([2, 4, 6]))
 
@@ -184,8 +200,9 @@ def test_apply_gufunc_elemwise_01():
 def test_apply_gufunc_elemwise_01b():
     def add(x, y):
         return x + y
-    a = da.from_array(np.array([1, 2, 3]), chunks=2, name='a')
-    b = da.from_array(np.array([1, 2, 3]), chunks=1, name='b')
+
+    a = da.from_array(np.array([1, 2, 3]), chunks=2, name="a")
+    b = da.from_array(np.array([1, 2, 3]), chunks=1, name="b")
     with pytest.raises(ValueError):
         apply_gufunc(add, "(),()->()", a, b, output_dtypes=a.dtype)
 
@@ -194,8 +211,9 @@ def test_apply_gufunc_elemwise_02():
     def addmul(x, y):
         assert x.shape in ((2,), (1,))
         return x + y, x * y
-    a = da.from_array(np.array([1, 2, 3]), chunks=2, name='a')
-    b = da.from_array(np.array([1, 2, 3]), chunks=2, name='b')
+
+    a = da.from_array(np.array([1, 2, 3]), chunks=2, name="a")
+    b = da.from_array(np.array([1, 2, 3]), chunks=2, name="b")
     z1, z2 = apply_gufunc(addmul, "(),()->(),()", a, b, output_dtypes=2 * (a.dtype,))
     assert_eq(z1, np.array([2, 4, 6]))
     assert_eq(z2, np.array([1, 4, 9]))
@@ -204,6 +222,7 @@ def test_apply_gufunc_elemwise_02():
 def test_gufunc_vector_output():
     def foo():
         return np.array([1, 2, 3], dtype=int)
+
     x = apply_gufunc(foo, "->(i_0)", output_dtypes=int, output_sizes={"i_0": 3})
     assert x.chunks == ((3,),)
     assert_eq(x, np.array([1, 2, 3]))
@@ -213,7 +232,8 @@ def test_apply_gufunc_elemwise_loop():
     def foo(x):
         assert x.shape in ((2,), (1,))
         return 2 * x
-    a = da.from_array(np.array([1, 2, 3]), chunks=2, name='a')
+
+    a = da.from_array(np.array([1, 2, 3]), chunks=2, name="a")
     z = apply_gufunc(foo, "()->()", a, output_dtypes=int)
     assert z.chunks == ((2, 1),)
     assert_eq(z, np.array([2, 4, 6]))
@@ -223,7 +243,8 @@ def test_apply_gufunc_elemwise_core():
     def foo(x):
         assert x.shape == (3,)
         return 2 * x
-    a = da.from_array(np.array([1, 2, 3]), chunks=3, name='a')
+
+    a = da.from_array(np.array([1, 2, 3]), chunks=3, name="a")
     z = apply_gufunc(foo, "(i)->(i)", a, output_dtypes=int)
     assert z.chunks == ((3,),)
     assert_eq(z, np.array([2, 4, 6]))
@@ -240,6 +261,7 @@ def test_apply_gufunc_elemwise_core():
 def test_apply_gufunc_two_scalar_output():
     def foo():
         return 1, 2
+
     x, y = apply_gufunc(foo, "->(),()", output_dtypes=(int, int))
     assert x.compute() == 1
     assert y.compute() == 2
@@ -248,28 +270,31 @@ def test_apply_gufunc_two_scalar_output():
 def test_apply_gufunc_two_mixed_outputs():
     def foo():
         return 1, np.ones((2, 3), dtype=float)
-    x, y = apply_gufunc(foo, "->(),(i,j)",
-                        output_dtypes=(int, float),
-                        output_sizes={'i': 2, 'j': 3})
+
+    x, y = apply_gufunc(
+        foo, "->(),(i,j)", output_dtypes=(int, float), output_sizes={"i": 2, "j": 3}
+    )
     assert x.compute() == 1
     assert y.chunks == ((2,), (3,))
     assert_eq(y, np.ones((2, 3), dtype=float))
 
 
-@pytest.mark.parametrize('output_dtypes', [int, (int,)])
+@pytest.mark.parametrize("output_dtypes", [int, (int,)])
 def test_apply_gufunc_output_dtypes(output_dtypes):
     def foo(x):
         return y
+
     x = np.random.randn(10)
     y = x.astype(int)
     dy = apply_gufunc(foo, "()->()", x, output_dtypes=output_dtypes)
-    #print(x, x.compute())
+    # print(x, x.compute())
     assert_eq(y, dy)
 
 
 def test_gufunc_two_inputs():
     def foo(x, y):
-        return np.einsum('...ij,...jk->ik', x, y)
+        return np.einsum("...ij,...jk->ik", x, y)
+
     a = da.ones((2, 3), chunks=100, dtype=int)
     b = da.ones((3, 4), chunks=100, dtype=int)
     x = apply_gufunc(foo, "(i,j),(j,k)->(i,k)", a, b, output_dtypes=int)
@@ -279,6 +304,7 @@ def test_gufunc_two_inputs():
 def test_gufunc_mixed_inputs():
     def foo(x, y):
         return x + y
+
     a = np.ones((2, 1), dtype=int)
     b = da.ones((1, 8), chunks=(2, 3), dtype=int)
     x = apply_gufunc(foo, "(),()->()", a, b, output_dtypes=int)
@@ -290,7 +316,15 @@ def test_gufunc():
 
     def foo(x):
         return np.mean(x, axis=-1)
-    gufoo = gufunc(foo, signature="(i)->()", axis=-1, keepdims=False, output_dtypes=float, vectorize=True)
+
+    gufoo = gufunc(
+        foo,
+        signature="(i)->()",
+        axis=-1,
+        keepdims=False,
+        output_dtypes=float,
+        vectorize=True,
+    )
 
     y = gufoo(x)
     valy = y.compute()
@@ -318,10 +352,12 @@ def test_apply_gufunc_broadcasting_loopdims():
         x, y = np.broadcast_arrays(x, y)
         return x, y, x * y
 
-    a = da.random.normal(size=(    10, 30), chunks=(8, 30))
-    b = da.random.normal(size=(20,  1, 30), chunks=(3, 1, 30))
+    a = da.random.normal(size=(10, 30), chunks=(8, 30))
+    b = da.random.normal(size=(20, 1, 30), chunks=(3, 1, 30))
 
-    x, y, z = apply_gufunc(foo, "(i),(i)->(i),(i),(i)", a, b, output_dtypes=3 * (float,), vectorize=False)
+    x, y, z = apply_gufunc(
+        foo, "(i),(i)->(i),(i),(i)", a, b, output_dtypes=3 * (float,), vectorize=False
+    )
 
     assert x.compute().shape == (20, 10, 30)
     assert y.compute().shape == (20, 10, 30)
@@ -343,6 +379,7 @@ def test_apply_gufunc_check_same_dimsizes():
 def test_apply_gufunc_check_coredim_chunksize():
     def foo(x):
         return np.sum(x, axis=-1)
+
     a = da.random.normal(size=(8,), chunks=3)
     with pytest.raises(ValueError) as excinfo:
         da.apply_gufunc(foo, "(i)->()", a, output_dtypes=float, allow_rechunk=False)
@@ -357,7 +394,9 @@ def test_apply_gufunc_check_inhomogeneous_chunksize():
     b = da.random.normal(size=(8,), chunks=((2, 3, 3),))
 
     with pytest.raises(ValueError) as excinfo:
-        da.apply_gufunc(foo, "(),()->()", a, b, output_dtypes=float, allow_rechunk=False)
+        da.apply_gufunc(
+            foo, "(),()->()", a, b, output_dtypes=float, allow_rechunk=False
+        )
     assert "with different chunksize present" in str(excinfo.value)
 
 
@@ -368,19 +407,19 @@ def test_apply_gufunc_infer_dtype():
     dy = da.from_array(y, chunks=5)
 
     def foo(x, *args, **kwargs):
-        cast = kwargs.pop('cast', 'i8')
+        cast = kwargs.pop("cast", "i8")
         return (x + sum(args)).astype(cast)
 
     dz = apply_gufunc(foo, "(),(),()->()", dx, dy, 1)
     z = foo(dx, dy, 1)
     assert_eq(dz, z)
 
-    dz = apply_gufunc(foo, "(),(),()->()", dx, dy, 1, cast='f8')
-    z = foo(dx, dy, 1, cast='f8')
+    dz = apply_gufunc(foo, "(),(),()->()", dx, dy, 1, cast="f8")
+    z = foo(dx, dy, 1, cast="f8")
     assert_eq(dz, z)
 
-    dz = apply_gufunc(foo, "(),(),()->()", dx, dy, 1, cast='f8', output_dtypes='f8')
-    z = foo(dx, dy, 1, cast='f8')
+    dz = apply_gufunc(foo, "(),(),()->()", dx, dy, 1, cast="f8", output_dtypes="f8")
+    z = foo(dx, dy, 1, cast="f8")
     assert_eq(dz, z)
 
     def foo(x):
@@ -391,7 +430,7 @@ def test_apply_gufunc_infer_dtype():
     msg = str(e.value)
     assert msg.startswith("`dtype` inference failed")
     assert "Please specify the dtype explicitly" in msg
-    assert 'RuntimeError' in msg
+    assert "RuntimeError" in msg
 
     # Multiple outputs
     def foo(x, y):
@@ -403,7 +442,7 @@ def test_apply_gufunc_infer_dtype():
     assert_eq(z1, dx - dy)
 
 
-@pytest.mark.parametrize('keepdims', [False, True])
+@pytest.mark.parametrize("keepdims", [False, True])
 def test_apply_gufunc_axis_01(keepdims):
     def mymedian(x):
         return np.median(x, axis=-1)
@@ -412,7 +451,9 @@ def test_apply_gufunc_axis_01(keepdims):
     da_ = da.from_array(a, chunks=2)
 
     m = np.median(a, axis=0, keepdims=keepdims)
-    dm = apply_gufunc(mymedian, "(i)->()", da_, axis=0, keepdims=keepdims, allow_rechunk=True)
+    dm = apply_gufunc(
+        mymedian, "(i)->()", da_, axis=0, keepdims=keepdims, allow_rechunk=True
+    )
     assert_eq(m, dm)
 
 
@@ -451,11 +492,13 @@ def test_apply_gufunc_axis_03():
     da_ = da.from_array(a, chunks=2)
 
     m = np.diff(a, axis=1)
-    dm = apply_gufunc(mydiff, "(i)->(i)", da_, axis=1, output_sizes={'i': 5}, allow_rechunk=True)
+    dm = apply_gufunc(
+        mydiff, "(i)->(i)", da_, axis=1, output_sizes={"i": 5}, allow_rechunk=True
+    )
     assert_eq(m, dm)
 
 
-@pytest.mark.parametrize('axis', [-2, -1, None])
+@pytest.mark.parametrize("axis", [-2, -1, None])
 def test_apply_gufunc_axis_keepdims(axis):
     def mymedian(x):
         return np.median(x, axis=-1)
@@ -464,11 +507,13 @@ def test_apply_gufunc_axis_keepdims(axis):
     da_ = da.from_array(a, chunks=2)
 
     m = np.median(a, axis=-1 if not axis else axis, keepdims=True)
-    dm = apply_gufunc(mymedian, "(i)->()", da_, axis=axis, keepdims=True, allow_rechunk=True)
+    dm = apply_gufunc(
+        mymedian, "(i)->()", da_, axis=axis, keepdims=True, allow_rechunk=True
+    )
     assert_eq(m, dm)
 
 
-@pytest.mark.parametrize('axes', [[0, 1], [(0,), (1,)]])
+@pytest.mark.parametrize("axes", [[0, 1], [(0,), (1,)]])
 def test_apply_gufunc_axes_01(axes):
     def mystats(x, y):
         return np.std(x, axis=-1) * np.mean(y, axis=-1)
@@ -494,12 +539,19 @@ def test_apply_gufunc_axes_02():
     db = da.from_array(b, chunks=3)
 
     m = np.einsum("jiu,juk->uik", a, b)
-    dm = apply_gufunc(matmul, "(i,j),(j,k)->(i,k)", da_, db, axes=[(1,  0), (0, -1), (-2, -1)], allow_rechunk=True)
+    dm = apply_gufunc(
+        matmul,
+        "(i,j),(j,k)->(i,k)",
+        da_,
+        db,
+        axes=[(1, 0), (0, -1), (-2, -1)],
+        allow_rechunk=True,
+    )
     assert_eq(m, dm)
 
 
 def test_apply_gufunc_axes_two_kept_coredims():
-    a = da.random.normal(size=(   20, 30), chunks=(10, 30))
+    a = da.random.normal(size=(20, 30), chunks=(10, 30))
     b = da.random.normal(size=(10, 1, 40), chunks=(5, 1, 40))
 
     def outer_product(x, y):
@@ -510,9 +562,11 @@ def test_apply_gufunc_axes_two_kept_coredims():
 
 
 def test_apply_gufunc_via_numba_01():
-    numba = pytest.importorskip('numba')
+    numba = pytest.importorskip("numba")
 
-    @numba.guvectorize([(numba.float64[:], numba.float64[:], numba.float64[:])], '(n),(n)->(n)')
+    @numba.guvectorize(
+        [(numba.float64[:], numba.float64[:], numba.float64[:])], "(n),(n)->(n)"
+    )
     def g(x, y, res):
         for i in range(x.shape[0]):
             res[i] = x[i] + y[i]
@@ -527,11 +581,11 @@ def test_apply_gufunc_via_numba_01():
 
 
 def test_apply_gufunc_via_numba_02():
-    numba = pytest.importorskip('numba')
+    numba = pytest.importorskip("numba")
 
-    @numba.guvectorize([(numba.float64[:], numba.float64[:])], '(n)->()')
+    @numba.guvectorize([(numba.float64[:], numba.float64[:])], "(n)->()")
     def mysum(x, res):
-        res[0] = 0.
+        res[0] = 0.0
         for i in range(x.shape[0]):
             res[0] += x[i]
 
@@ -541,3 +595,24 @@ def test_apply_gufunc_via_numba_02():
     y = mysum(a, axis=0, keepdims=True, allow_rechunk=True)
 
     assert_eq(x, y)
+
+
+@pytest.mark.skipif(
+    not IS_NEP18_ACTIVE, reason="NEP18 required for sparse meta propagation"
+)
+def test_preserve_meta_type():
+    sparse = pytest.importorskip("sparse")
+
+    def stats(x):
+        return np.sum(x, axis=-1), np.mean(x, axis=-1)
+
+    a = da.random.normal(size=(10, 20, 30), chunks=(5, 5, 30))
+    a = a.map_blocks(sparse.COO.from_numpy)
+    sum, mean = apply_gufunc(stats, "(i)->(),()", a, output_dtypes=2 * (a.dtype,))
+
+    assert isinstance(a._meta, sparse.COO)
+    assert isinstance(sum._meta, sparse.COO)
+    assert isinstance(mean._meta, sparse.COO)
+
+    assert_eq(sum, sum)
+    assert_eq(mean, mean)
