@@ -2,7 +2,8 @@ from itertools import product
 import warnings
 
 import pytest
-np = pytest.importorskip('numpy')
+
+np = pytest.importorskip("numpy")
 
 import dask
 from dask.utils import funcname
@@ -16,54 +17,64 @@ import dask.array as da
 def test_rechunk_internals_1():
     """ Test the cumdims_label and _breakpoints and
     _intersect_1d internal funcs to rechunk."""
-    new = cumdims_label(((1,1,2),(1,5,1)),'n')
-    old = cumdims_label(((4, ),(1,) * 5),'o')
+    new = cumdims_label(((1, 1, 2), (1, 5, 1)), "n")
+    old = cumdims_label(((4,), (1,) * 5), "o")
     breaks = tuple(_breakpoints(o, n) for o, n in zip(old, new))
-    answer = (('o', 0), ('n', 0), ('n', 1), ('n', 2), ('o', 4), ('n', 4))
+    answer = (("o", 0), ("n", 0), ("n", 1), ("n", 2), ("o", 4), ("n", 4))
     assert breaks[0] == answer
-    answer2 = (('o', 0), ('n', 0), ('o', 1), ('n', 1), ('o', 2),
-               ('o', 3), ('o', 4), ('o', 5), ('n', 6), ('n', 7))
+    answer2 = (
+        ("o", 0),
+        ("n", 0),
+        ("o", 1),
+        ("n", 1),
+        ("o", 2),
+        ("o", 3),
+        ("o", 4),
+        ("o", 5),
+        ("n", 6),
+        ("n", 7),
+    )
     assert breaks[1] == answer2
     i1d = [_intersect_1d(b) for b in breaks]
-    answer3 = [[(0, slice(0, 1))],
-               [(0, slice(1, 2))],
-               [(0, slice(2, 4))]]
+    answer3 = [[(0, slice(0, 1))], [(0, slice(1, 2))], [(0, slice(2, 4))]]
     assert i1d[0] == answer3
-    answer4 = [[(0, slice(0, 1))],
-               [(1, slice(0, 1)),
-                (2, slice(0, 1)),
-                (3, slice(0, 1)),
-                (4, slice(0, 1)),
-                (5, slice(0, 1))],
-               [(5, slice(1, 2))]]
+    answer4 = [
+        [(0, slice(0, 1))],
+        [
+            (1, slice(0, 1)),
+            (2, slice(0, 1)),
+            (3, slice(0, 1)),
+            (4, slice(0, 1)),
+            (5, slice(0, 1)),
+        ],
+        [(5, slice(1, 2))],
+    ]
     assert i1d[1] == answer4
 
 
 def test_intersect_1():
     """ Convert 1 D chunks"""
-    old = ((10, 10, 10, 10, 10), )
-    new = ((25, 5, 20), )
-    answer = [(((0, slice(0, 10)), ),
-              ((1, slice(0, 10)), ),
-              ((2, slice(0, 5)), )),
-              (((2, slice(5, 10)), ), ),
-              (((3, slice(0, 10)), ), ((4, slice(0, 10)), ))
-              ]
+    old = ((10, 10, 10, 10, 10),)
+    new = ((25, 5, 20),)
+    answer = [
+        (((0, slice(0, 10)),), ((1, slice(0, 10)),), ((2, slice(0, 5)),)),
+        (((2, slice(5, 10)),),),
+        (((3, slice(0, 10)),), ((4, slice(0, 10)),)),
+    ]
     cross = list(intersect_chunks(old_chunks=old, new_chunks=new))
     assert answer == cross
 
 
 def test_intersect_2():
     """ Convert 1 D chunks"""
-    old = ((20, 20, 20, 20, 20), )
+    old = ((20, 20, 20, 20, 20),)
     new = ((58, 4, 20, 18),)
-    answer = [(((0, slice(0, 20)), ),
-              ((1, slice(0, 20)), ),
-              ((2, slice(0, 18)), )),
-              (((2, slice(18, 20)), ), ((3, slice(0, 2)), )),
-              (((3, slice(2, 20)), ), ((4, slice(0, 2)), )),
-              (((4, slice(2, 20)), ), )
-              ]
+    answer = [
+        (((0, slice(0, 20)),), ((1, slice(0, 20)),), ((2, slice(0, 18)),)),
+        (((2, slice(18, 20)),), ((3, slice(0, 2)),)),
+        (((3, slice(2, 20)),), ((4, slice(0, 2)),)),
+        (((4, slice(2, 20)),),),
+    ]
     cross = list(intersect_chunks(old_chunks=old, new_chunks=new))
     assert answer == cross
 
@@ -71,8 +82,8 @@ def test_intersect_2():
 def test_rechunk_1d():
     """Try rechunking a random 1d matrix"""
     a = np.random.uniform(0, 1, 30)
-    x = da.from_array(a, chunks=((10, ) * 3, ))
-    new = ((5, ) * 6,)
+    x = da.from_array(a, chunks=((10,) * 3,))
+    new = ((5,) * 6,)
     x2 = rechunk(x, chunks=new)
     assert x2.chunks == new
     assert np.all(x2.compute() == a)
@@ -81,8 +92,8 @@ def test_rechunk_1d():
 def test_rechunk_2d():
     """Try rechunking a random 2d matrix"""
     a = np.random.uniform(0, 1, 300).reshape((10, 30))
-    x = da.from_array(a, chunks=((1, 2, 3, 4), (5, ) * 6))
-    new = ((5, 5), (15, ) * 2)
+    x = da.from_array(a, chunks=((1, 2, 3, 4), (5,) * 6))
+    new = ((5, 5), (15,) * 2)
     x2 = rechunk(x, chunks=new)
     assert x2.chunks == new
     assert np.all(x2.compute() == a)
@@ -90,10 +101,10 @@ def test_rechunk_2d():
 
 def test_rechunk_4d():
     """Try rechunking a random 4d matrix"""
-    old = ((5, 5), ) * 4
-    a = np.random.uniform(0, 1, 10000).reshape((10, ) * 4)
+    old = ((5, 5),) * 4
+    a = np.random.uniform(0, 1, 10000).reshape((10,) * 4)
     x = da.from_array(a, chunks=old)
-    new = ((10, ), ) * 4
+    new = ((10,),) * 4
     x2 = rechunk(x, chunks=new)
     assert x2.chunks == new
     assert np.all(x2.compute() == a)
@@ -110,21 +121,21 @@ def test_rechunk_expand2():
     (a, b) = (3, 2)
     orig = np.random.uniform(0, 1, a ** b).reshape((a,) * b)
     for off, off2 in product(range(1, a - 1), range(1, a - 1)):
-        old = ((a - off, off), ) * b
+        old = ((a - off, off),) * b
         x = da.from_array(orig, chunks=old)
-        new = ((a - off2, off2), ) * b
+        new = ((a - off2, off2),) * b
         assert np.all(x.rechunk(chunks=new).compute() == orig)
         if a - off - off2 > 0:
-            new = ((off, a - off2 - off, off2), ) * b
+            new = ((off, a - off2 - off, off2),) * b
             y = x.rechunk(chunks=new).compute()
             assert np.all(y == orig)
 
 
 def test_rechunk_method():
     """ Test rechunking can be done as a method of dask array."""
-    old = ((5, 2, 3), ) * 4
-    new = ((3, 3, 3, 1), ) * 4
-    a = np.random.uniform(0, 1, 10000).reshape((10, ) * 4)
+    old = ((5, 2, 3),) * 4
+    new = ((3, 3, 3, 1),) * 4
+    a = np.random.uniform(0, 1, 10000).reshape((10,) * 4)
     x = da.from_array(a, chunks=old)
     x2 = x.rechunk(chunks=new)
     assert x2.chunks == new
@@ -136,7 +147,7 @@ def test_rechunk_blockshape():
     new_shape, new_chunks = (10, 10), (4, 3)
     new_blockdims = normalize_chunks(new_chunks, new_shape)
     old_chunks = ((4, 4, 2), (3, 3, 3, 1))
-    a = np.random.uniform(0,1,100).reshape((10, 10))
+    a = np.random.uniform(0, 1, 100).reshape((10, 10))
     x = da.from_array(a, chunks=old_chunks)
     check1 = rechunk(x, chunks=new_chunks)
     assert check1.chunks == new_blockdims
@@ -170,8 +181,7 @@ def test_rechunk_with_empty_input():
 
 def test_rechunk_with_null_dimensions():
     x = da.from_array(np.ones((24, 24)), chunks=(4, 8))
-    assert (x.rechunk(chunks=(None, 4)).chunks ==
-            da.ones((24, 24), chunks=(4, 4)).chunks)
+    assert x.rechunk(chunks=(None, 4)).chunks == da.ones((24, 24), chunks=(4, 4)).chunks
 
 
 def test_rechunk_with_integer():
@@ -227,13 +237,7 @@ def test_divide_to_width():
     assert chunks == (8, 9, 10)
     chunks = divide_to_width((8, 2, 9, 10, 11, 12), 4)
     # Note how 9 gives (3, 3, 3), not (4, 4, 1) or whatever
-    assert chunks == (4, 4,
-                      2,
-                      3, 3, 3,
-                      3, 3, 4,
-                      3, 4, 4,
-                      4, 4, 4,
-                      )
+    assert chunks == (4, 4, 2, 3, 3, 3, 3, 3, 4, 3, 4, 4, 4, 4, 4)
 
 
 def test_merge_to_number():
@@ -277,9 +281,9 @@ def test_merge_to_number():
 
 
 def _plan(old_chunks, new_chunks, itemsize=1, block_size_limit=1e7):
-    return plan_rechunk(old_chunks, new_chunks,
-                        itemsize=itemsize,
-                        block_size_limit=block_size_limit)
+    return plan_rechunk(
+        old_chunks, new_chunks, itemsize=itemsize, block_size_limit=block_size_limit
+    )
 
 
 def _assert_steps(steps, expected):
@@ -288,10 +292,10 @@ def _assert_steps(steps, expected):
 
 
 def test_plan_rechunk():
-    c = ((20,) * 2)              # coarse
-    f = ((2,) * 20)              # fine
-    nc = ((float('nan'),) * 2)   # nan-coarse
-    nf = ((float('nan'),) * 20)  # nan-fine
+    c = (20,) * 2  # coarse
+    f = (2,) * 20  # fine
+    nc = (float("nan"),) * 2  # nan-coarse
+    nf = (float("nan"),) * 20  # nan-fine
 
     # Trivial cases
     steps = _plan((), ())
@@ -327,7 +331,7 @@ def test_plan_rechunk():
     _assert_steps(steps, [(c, c), (c, f)])
 
     # Hitting the memory limit => partial merge
-    m = ((10,) * 4)   # mid
+    m = (10,) * 4  # mid
 
     steps = _plan((f, c), (c, f), block_size_limit=399)
     _assert_steps(steps, [(m, c), (c, f)])
@@ -336,8 +340,8 @@ def test_plan_rechunk():
     _assert_steps(steps2, steps)
 
     # Larger problem size => more intermediates
-    c = ((1000,) * 2)   # coarse
-    f = ((2,) * 1000)   # fine
+    c = (1000,) * 2  # coarse
+    f = (2,) * 1000  # fine
 
     steps = _plan((f, c), (c, f), block_size_limit=99999)
     assert len(steps) == 3
@@ -352,8 +356,8 @@ def test_plan_rechunk():
 
 def test_plan_rechunk_5d():
     # 5d problem
-    c = ((10,) * 1)    # coarse
-    f = ((1,) * 10)    # fine
+    c = (10,) * 1  # coarse
+    f = (1,) * 10  # fine
 
     steps = _plan((c, c, c, c, c), (f, f, f, f, f))
     _assert_steps(steps, [(f, f, f, f, f)])
@@ -365,8 +369,8 @@ def test_plan_rechunk_5d():
 
 
 def test_plan_rechunk_heterogenous():
-    c = ((10,) * 1)    # coarse
-    f = ((1,) * 10)    # fine
+    c = (10,) * 1  # coarse
+    f = (1,) * 10  # fine
     cf = c + f
     cc = c + c
     ff = f + f
@@ -414,46 +418,50 @@ def test_rechunk_warning():
     assert not w
 
 
-@pytest.mark.parametrize('shape,chunks', [[(4,), (2,)],
-                                          [(4, 4), (2, 2)],
-                                          [(4, 4), (4, 2)]])
+@pytest.mark.parametrize(
+    "shape,chunks", [[(4,), (2,)], [(4, 4), (2, 2)], [(4, 4), (4, 2)]]
+)
 def test_dont_concatenate_single_chunks(shape, chunks):
     x = da.ones(shape, chunks=shape)
     y = x.rechunk(chunks)
     dsk = dict(y.dask)
-    assert not any(funcname(task[0]).startswith('concat')
-                   for task in dsk.values()
-                   if dask.istask(task))
+    assert not any(
+        funcname(task[0]).startswith("concat")
+        for task in dsk.values()
+        if dask.istask(task)
+    )
 
 
 def test_intersect_nan():
-    old_chunks = ((float('nan'), float('nan')), (8,))
-    new_chunks = ((float('nan'), float('nan')), (4, 4))
+    old_chunks = ((float("nan"), float("nan")), (8,))
+    new_chunks = ((float("nan"), float("nan")), (4, 4))
 
     result = list(intersect_chunks(old_chunks, new_chunks))
     expected = [
         (((0, slice(0, None, None)), (0, slice(0, 4, None))),),
         (((0, slice(0, None, None)), (0, slice(4, 8, None))),),
         (((1, slice(0, None, None)), (0, slice(0, 4, None))),),
-        (((1, slice(0, None, None)), (0, slice(4, 8, None))),)
+        (((1, slice(0, None, None)), (0, slice(4, 8, None))),),
     ]
     assert result == expected
 
 
 def test_intersect_nan_single():
-    old_chunks = ((float('nan'),), (10,))
-    new_chunks = ((float('nan'),), (5, 5))
+    old_chunks = ((float("nan"),), (10,))
+    new_chunks = ((float("nan"),), (5, 5))
 
     result = list(intersect_chunks(old_chunks, new_chunks))
-    expected = [(((0, slice(0, None, None)), (0, slice(0, 5, None))),),
-                (((0, slice(0, None, None)), (0, slice(5, 10, None))),)]
+    expected = [
+        (((0, slice(0, None, None)), (0, slice(0, 5, None))),),
+        (((0, slice(0, None, None)), (0, slice(5, 10, None))),),
+    ]
     assert result == expected
 
 
 def test_intersect_nan_long():
 
-    old_chunks = (tuple([float('nan')] * 4), (10,))
-    new_chunks = (tuple([float('nan')] * 4), (5, 5))
+    old_chunks = (tuple([float("nan")] * 4), (10,))
+    new_chunks = (tuple([float("nan")] * 4), (5, 5))
     result = list(intersect_chunks(old_chunks, new_chunks))
     expected = [
         (((0, slice(0, None, None)), (0, slice(0, 5, None))),),
@@ -463,14 +471,14 @@ def test_intersect_nan_long():
         (((2, slice(0, None, None)), (0, slice(0, 5, None))),),
         (((2, slice(0, None, None)), (0, slice(5, 10, None))),),
         (((3, slice(0, None, None)), (0, slice(0, 5, None))),),
-        (((3, slice(0, None, None)), (0, slice(5, 10, None))),)
+        (((3, slice(0, None, None)), (0, slice(5, 10, None))),),
     ]
     assert result == expected
 
 
 def test_rechunk_unknown_from_pandas():
-    dd = pytest.importorskip('dask.dataframe')
-    pd = pytest.importorskip('pandas')
+    dd = pytest.importorskip("dask.dataframe")
+    pd = pytest.importorskip("pandas")
 
     arr = np.random.randn(50, 10)
     x = dd.from_pandas(pd.DataFrame(arr), 2).values
@@ -483,7 +491,7 @@ def test_rechunk_unknown_from_pandas():
 
 
 def test_rechunk_unknown_from_array():
-    dd = pytest.importorskip('dask.dataframe')
+    dd = pytest.importorskip("dask.dataframe")
     # pd = pytest.importorskip('pandas')
     x = dd.from_array(da.ones(shape=(4, 4), chunks=(2, 2))).values
     # result = x.rechunk({1: 5})
@@ -494,25 +502,25 @@ def test_rechunk_unknown_from_array():
     assert_eq(x, result)
 
 
-@pytest.mark.parametrize('x, chunks', [
-    (da.ones(shape=(50, 10), chunks=(25, 10)), (None, 5)),
-    (da.ones(shape=(50, 10), chunks=(25, 10)), {1: 5}),
-    (da.ones(shape=(50, 10), chunks=(25, 10)), (None, (5, 5))),
-
-    (da.ones(shape=(1000, 10), chunks=(5, 10)), (None, 5)),
-    (da.ones(shape=(1000, 10), chunks=(5, 10)), {1: 5}),
-    (da.ones(shape=(1000, 10), chunks=(5, 10)), (None, (5, 5))),
-
-    (da.ones(shape=(10, 10), chunks=(10, 10)), (None, 5)),
-    (da.ones(shape=(10, 10), chunks=(10, 10)), {1: 5}),
-    (da.ones(shape=(10, 10), chunks=(10, 10)), (None, (5, 5))),
-
-    (da.ones(shape=(10, 10), chunks=(10, 2)), (None, 5)),
-    (da.ones(shape=(10, 10), chunks=(10, 2)), {1: 5}),
-    (da.ones(shape=(10, 10), chunks=(10, 2)), (None, (5, 5))),
-])
+@pytest.mark.parametrize(
+    "x, chunks",
+    [
+        (da.ones(shape=(50, 10), chunks=(25, 10)), (None, 5)),
+        (da.ones(shape=(50, 10), chunks=(25, 10)), {1: 5}),
+        (da.ones(shape=(50, 10), chunks=(25, 10)), (None, (5, 5))),
+        (da.ones(shape=(1000, 10), chunks=(5, 10)), (None, 5)),
+        (da.ones(shape=(1000, 10), chunks=(5, 10)), {1: 5}),
+        (da.ones(shape=(1000, 10), chunks=(5, 10)), (None, (5, 5))),
+        (da.ones(shape=(10, 10), chunks=(10, 10)), (None, 5)),
+        (da.ones(shape=(10, 10), chunks=(10, 10)), {1: 5}),
+        (da.ones(shape=(10, 10), chunks=(10, 10)), (None, (5, 5))),
+        (da.ones(shape=(10, 10), chunks=(10, 2)), (None, 5)),
+        (da.ones(shape=(10, 10), chunks=(10, 2)), {1: 5}),
+        (da.ones(shape=(10, 10), chunks=(10, 2)), (None, (5, 5))),
+    ],
+)
 def test_rechunk_unknown(x, chunks):
-    dd = pytest.importorskip('dask.dataframe')
+    dd = pytest.importorskip("dask.dataframe")
     y = dd.from_array(x).values
     result = y.rechunk(chunks)
     expected = x.rechunk(chunks)
@@ -522,10 +530,10 @@ def test_rechunk_unknown(x, chunks):
 
 
 def test_rechunk_unknown_explicit():
-    dd = pytest.importorskip('dask.dataframe')
+    dd = pytest.importorskip("dask.dataframe")
     x = da.ones(shape=(10, 10), chunks=(5, 2))
     y = dd.from_array(x).values
-    result = y.rechunk(((float('nan'), float('nan')), (5, 5)))
+    result = y.rechunk(((float("nan"), float("nan")), (5, 5)))
     expected = x.rechunk((None, (5, 5)))
     assert_chunks_match(result.chunks, expected.chunks)
     assert_eq(result, expected)
@@ -540,7 +548,7 @@ def assert_chunks_match(left, right):
 
 
 def test_rechunk_unknown_raises():
-    dd = pytest.importorskip('dask.dataframe')
+    dd = pytest.importorskip("dask.dataframe")
 
     x = dd.from_array(da.ones(shape=(10, 10), chunks=(5, 5))).values
     with pytest.raises(ValueError):
@@ -548,59 +556,71 @@ def test_rechunk_unknown_raises():
 
 
 def test_old_to_new_single():
-    old = ((float('nan'), float('nan')), (8,))
-    new = ((float('nan'), float('nan')), (4, 4))
+    old = ((float("nan"), float("nan")), (8,))
+    new = ((float("nan"), float("nan")), (4, 4))
     result = _old_to_new(old, new)
 
-    expected = [[[(0, slice(0, None, None))], [(1, slice(0, None, None))]],
-                [[(0, slice(0, 4, None))], [(0, slice(4, 8, None))]]]
+    expected = [
+        [[(0, slice(0, None, None))], [(1, slice(0, None, None))]],
+        [[(0, slice(0, 4, None))], [(0, slice(4, 8, None))]],
+    ]
 
     assert result == expected
 
 
 def test_old_to_new():
-    old = ((float('nan'),), (10,))
-    new = ((float('nan'),), (5, 5))
+    old = ((float("nan"),), (10,))
+    new = ((float("nan"),), (5, 5))
     result = _old_to_new(old, new)
-    expected = [[[(0, slice(0, None, None))]],
-                [[(0, slice(0, 5, None))], [(0, slice(5, 10, None))]]]
+    expected = [
+        [[(0, slice(0, None, None))]],
+        [[(0, slice(0, 5, None))], [(0, slice(5, 10, None))]],
+    ]
 
     assert result == expected
 
 
 def test_old_to_new_large():
-    old = (tuple([float('nan')] * 4), (10,))
-    new = (tuple([float('nan')] * 4), (5, 5))
+    old = (tuple([float("nan")] * 4), (10,))
+    new = (tuple([float("nan")] * 4), (5, 5))
 
     result = _old_to_new(old, new)
-    expected = [[[(0, slice(0, None, None))],
-                [(1, slice(0, None, None))],
-                [(2, slice(0, None, None))],
-                [(3, slice(0, None, None))]],
-                [[(0, slice(0, 5, None))], [(0, slice(5, 10, None))]]]
+    expected = [
+        [
+            [(0, slice(0, None, None))],
+            [(1, slice(0, None, None))],
+            [(2, slice(0, None, None))],
+            [(3, slice(0, None, None))],
+        ],
+        [[(0, slice(0, 5, None))], [(0, slice(5, 10, None))]],
+    ]
     assert result == expected
 
 
 def test_changing_raises():
-    nan = float('nan')
+    nan = float("nan")
     with pytest.raises(ValueError) as record:
         _old_to_new(((nan, nan), (4, 4)), ((nan, nan, nan), (4, 4)))
 
-    assert 'unchanging' in str(record.value)
+    assert "unchanging" in str(record.value)
 
 
 def test_old_to_new_known():
-    old = ((10, 10, 10, 10, 10), )
-    new = ((25, 5, 20), )
+    old = ((10, 10, 10, 10, 10),)
+    new = ((25, 5, 20),)
     result = _old_to_new(old, new)
-    expected = [[[(0, slice(0, 10, None)), (1, slice(0, 10, None)), (2, slice(0, 5, None))],
-                [(2, slice(5, 10, None))],
-                [(3, slice(0, 10, None)), (4, slice(0, 10, None))]]]
+    expected = [
+        [
+            [(0, slice(0, 10, None)), (1, slice(0, 10, None)), (2, slice(0, 5, None))],
+            [(2, slice(5, 10, None))],
+            [(3, slice(0, 10, None)), (4, slice(0, 10, None))],
+        ]
+    ]
     assert result == expected
 
 
 def test_rechunk_zero_dim():
-    da = pytest.importorskip('dask.array')
+    da = pytest.importorskip("dask.array")
 
     x = da.ones((0, 10, 100), chunks=(0, 10, 10)).rechunk((0, 10, 50))
     assert len(x.compute()) == 0
@@ -613,16 +633,19 @@ def test_rechunk_avoid_needless_chunking():
     assert len(dsk) <= 8 + 2
 
 
-@pytest.mark.parametrize('shape,chunks,bs,expected', [
-    (100, 1, 10, (10,) * 10),
-    (100, 50, 10, (10,) * 10),
-    (100, 100, 10, (10,) * 10),
-    (20, 7, 10, (7, 7, 6)),
-    (20, (1, 1, 1, 1, 6, 2, 1, 7), 5, (5, 5, 5, 5)),
-])
+@pytest.mark.parametrize(
+    "shape,chunks,bs,expected",
+    [
+        (100, 1, 10, (10,) * 10),
+        (100, 50, 10, (10,) * 10),
+        (100, 100, 10, (10,) * 10),
+        (20, 7, 10, (7, 7, 6)),
+        (20, (1, 1, 1, 1, 6, 2, 1, 7), 5, (5, 5, 5, 5)),
+    ],
+)
 def test_rechunk_auto_1d(shape, chunks, bs, expected):
     x = da.ones(shape, chunks=(chunks,))
-    y = x.rechunk({0: 'auto'}, block_size_limit=bs * x.dtype.itemsize)
+    y = x.rechunk({0: "auto"}, block_size_limit=bs * x.dtype.itemsize)
     assert y.chunks == (expected,)
 
 
@@ -632,66 +655,87 @@ def test_rechunk_auto_2d():
     assert y.chunks == ((20,), (1,) * 20)
 
     x = da.ones((20, 20), chunks=(2, 2))
-    y = x.rechunk((-1, 'auto'), block_size_limit=80 * x.dtype.itemsize)
+    y = x.rechunk((-1, "auto"), block_size_limit=80 * x.dtype.itemsize)
     assert y.chunks == ((20,), (4,) * 5)
 
     x = da.ones((20, 20), chunks=((2, 2)))
-    y = x.rechunk({0: 'auto'}, block_size_limit=20 * x.dtype.itemsize)
+    y = x.rechunk({0: "auto"}, block_size_limit=20 * x.dtype.itemsize)
     assert y.chunks[1] == x.chunks[1]
     assert y.chunks[0] == (10, 10)
 
     x = da.ones((20, 20), chunks=((2,) * 10, (2, 2, 2, 2, 2, 5, 5)))
-    y = x.rechunk({0: 'auto'}, block_size_limit=20 * x.dtype.itemsize)
+    y = x.rechunk({0: "auto"}, block_size_limit=20 * x.dtype.itemsize)
     assert y.chunks[1] == x.chunks[1]
     assert y.chunks[0] == (4, 4, 4, 4, 4)  # limited by largest
 
 
 def test_rechunk_auto_3d():
     x = da.ones((20, 20, 20), chunks=((2, 2, 2)))
-    y = x.rechunk({0: 'auto', 1: 'auto'}, block_size_limit=200 * x.dtype.itemsize)
+    y = x.rechunk({0: "auto", 1: "auto"}, block_size_limit=200 * x.dtype.itemsize)
     assert y.chunks[2] == x.chunks[2]
     assert y.chunks[0] == (10, 10)
     assert y.chunks[1] == (10, 10)  # even split
 
 
-@pytest.mark.parametrize('n', [100, 1000])
+@pytest.mark.parametrize("n", [100, 1000])
 def test_rechunk_auto_image_stack(n):
-    with dask.config.set({'array.chunk-size': '10MiB'}):
-        x = da.ones((n, 1000, 1000), chunks=(1, 1000, 1000), dtype='uint8')
-        y = x.rechunk('auto')
+    with dask.config.set({"array.chunk-size": "10MiB"}):
+        x = da.ones((n, 1000, 1000), chunks=(1, 1000, 1000), dtype="uint8")
+        y = x.rechunk("auto")
         assert y.chunks == ((10,) * (n // 10), (1000,), (1000,))
-        assert y.rechunk('auto').chunks == y.chunks  # idempotent
+        assert y.rechunk("auto").chunks == y.chunks  # idempotent
 
-    with dask.config.set({'array.chunk-size': '7MiB'}):
-        z = x.rechunk('auto')
+    with dask.config.set({"array.chunk-size": "7MiB"}):
+        z = x.rechunk("auto")
         assert z.chunks == ((5,) * (n // 5), (1000,), (1000,))
 
-    with dask.config.set({'array.chunk-size': '1MiB'}):
-        x = da.ones((n, 1000, 1000), chunks=(1, 1000, 1000), dtype='float64')
-        z = x.rechunk('auto')
-        assert z.chunks == ((1,) * n , (250,) * 4, (250,) * 4)
+    with dask.config.set({"array.chunk-size": "1MiB"}):
+        x = da.ones((n, 1000, 1000), chunks=(1, 1000, 1000), dtype="float64")
+        z = x.rechunk("auto")
+        assert z.chunks == ((1,) * n, (250,) * 4, (250,) * 4)
 
 
 def test_rechunk_down():
-    with dask.config.set({'array.chunk-size': '10MiB'}):
-        x = da.ones((100, 1000, 1000), chunks=(1, 1000, 1000), dtype='uint8')
-        y = x.rechunk('auto')
+    with dask.config.set({"array.chunk-size": "10MiB"}):
+        x = da.ones((100, 1000, 1000), chunks=(1, 1000, 1000), dtype="uint8")
+        y = x.rechunk("auto")
         assert y.chunks == ((10,) * 10, (1000,), (1000,))
 
-    with dask.config.set({'array.chunk-size': '1MiB'}):
-        z = y.rechunk('auto')
+    with dask.config.set({"array.chunk-size": "1MiB"}):
+        z = y.rechunk("auto")
         assert z.chunks == ((5,) * 20, (250,) * 4, (250,) * 4)
 
-    with dask.config.set({'array.chunk-size': '1MiB'}):
-        z = y.rechunk({0: 'auto'})
-        assert z.chunks == ((1,) * 100, (1000,),  (1000,))
+    with dask.config.set({"array.chunk-size": "1MiB"}):
+        z = y.rechunk({0: "auto"})
+        assert z.chunks == ((1,) * 100, (1000,), (1000,))
 
-        z = y.rechunk({1: 'auto'})
-        assert z.chunks == ((10,) * 10, (100,) * 10,  (1000,))
+        z = y.rechunk({1: "auto"})
+        assert z.chunks == ((10,) * 10, (100,) * 10, (1000,))
 
 
 def test_rechunk_zero():
-    with dask.config.set({'array.chunk-size': '1B'}):
+    with dask.config.set({"array.chunk-size": "1B"}):
         x = da.ones(10, chunks=(5,))
-        y = x.rechunk('auto')
+        y = x.rechunk("auto")
         assert y.chunks == ((1,) * 10,)
+
+
+def test_rechunk_bad_keys():
+    x = da.zeros((2, 3, 4), chunks=1)
+    assert x.rechunk({-1: 4}).chunks == ((1, 1), (1, 1, 1), (4,))
+    assert x.rechunk({-x.ndim: 2}).chunks == ((2,), (1, 1, 1), (1, 1, 1, 1))
+
+    with pytest.raises(TypeError) as info:
+        x.rechunk({"blah": 4})
+
+    assert "blah" in str(info.value)
+
+    with pytest.raises(ValueError) as info:
+        x.rechunk({100: 4})
+
+    assert "100" in str(info.value)
+
+    with pytest.raises(ValueError) as info:
+        x.rechunk({-100: 4})
+
+    assert "-100" in str(info.value)
