@@ -615,10 +615,11 @@ def test_optimize(tmpdir, c):
 @write_read_engines()
 def test_roundtrip_from_pandas(tmpdir, write_engine, read_engine):
     fn = str(tmpdir.join("test.parquet"))
-    df = pd.DataFrame({"x": [1, 2, 3]})
-    df.to_parquet(fn, engine=write_engine)
-    ddf = dd.read_parquet(fn, engine=read_engine)
-    assert_eq(df, ddf)
+    dfp = df.copy()
+    dfp.index.name = "index"
+    dfp.to_parquet(fn, engine=write_engine)
+    ddf = dd.read_parquet(fn, index="index", engine=read_engine)
+    assert_eq(dfp, ddf)
 
 
 @write_read_engines_xfail
@@ -1041,6 +1042,8 @@ def test_partition_on(tmpdir, write_engine, read_engine):
 
 @write_read_engines_xfail
 def test_filters(tmpdir, write_engine, read_engine):
+    if write_engine == "fastparquet" or read_engine == "fastparquet":
+        pytest.importorskip("fastparquet", minversion="0.3.1")
     fn = str(tmpdir)
 
     df = pd.DataFrame({"at": ["ab", "aa", "ba", "da", "bb"]})
@@ -1073,7 +1076,7 @@ def test_filters(tmpdir, write_engine, read_engine):
 
 
 def test_divisions_read_with_filters(tmpdir):
-    check_fastparquet()
+    pytest.importorskip("fastparquet", minversion="0.3.1")
     tmpdir = str(tmpdir)
     # generate dataframe
     size = 100
@@ -1098,7 +1101,7 @@ def test_divisions_read_with_filters(tmpdir):
 
 
 def test_divisions_are_known_read_with_filters(tmpdir):
-    check_fastparquet()
+    pytest.importorskip("fastparquet", minversion="0.3.1")
     tmpdir = str(tmpdir)
     # generate dataframe
     df = pd.DataFrame(
