@@ -13,12 +13,15 @@ from .core import _emulate
 from .utils import make_meta, PANDAS_VERSION
 
 
-def overlap_chunk(func, prev_part, current_part, next_part, before, after,
-                  args, kwargs):
+def overlap_chunk(
+    func, prev_part, current_part, next_part, before, after, args, kwargs
+):
 
-    msg = ("Partition size is less than overlapping "
-           "window size. Try using ``df.repartition`` "
-           "to increase the partition size.")
+    msg = (
+        "Partition size is less than overlapping "
+        "window size. Try using ``df.repartition`` "
+        "to increase the partition size."
+    )
 
     if prev_part is not None and isinstance(before, Integral):
         if prev_part.shape[0] != before:
@@ -27,7 +30,6 @@ def overlap_chunk(func, prev_part, current_part, next_part, before, after,
     if next_part is not None and isinstance(after, Integral):
         if next_part.shape[0] != after:
             raise NotImplementedError(msg)
-    # We validate that the window isn't too large for tiemdeltas in map_overlap
 
     parts = [p for p in (prev_part, current_part, next_part) if p is not None]
     combined = pd.concat(parts)
@@ -66,62 +68,114 @@ def map_overlap(func, df, before, after, *args, **kwargs):
     --------
     dd.DataFrame.map_overlap
     """
-    if (isinstance(before, datetime.timedelta) or isinstance(after, datetime.timedelta)):
+    if isinstance(before, datetime.timedelta) or isinstance(after, datetime.timedelta):
         if not df.index._meta_nonempty.is_all_dates:
-            raise TypeError("Must have a `DatetimeIndex` when using string offset "
-                            "for `before` and `after`")
+            raise TypeError(
+                "Must have a `DatetimeIndex` when using string offset "
+                "for `before` and `after`"
+            )
     else:
-        if not (isinstance(before, Integral) and before >= 0 and
-                isinstance(after, Integral) and after >= 0):
+        if not (
+            isinstance(before, Integral)
+            and before >= 0
+            and isinstance(after, Integral)
+            and after >= 0
+        ):
             raise ValueError("before and after must be positive integers")
 
-    if 'token' in kwargs:
-        func_name = kwargs.pop('token')
+    if "token" in kwargs:
+        func_name = kwargs.pop("token")
         token = tokenize(df, before, after, *args, **kwargs)
     else:
-        func_name = 'overlap-' + funcname(func)
+        func_name = "overlap-" + funcname(func)
         token = tokenize(func, df, before, after, *args, **kwargs)
 
-    if 'meta' in kwargs:
-        meta = kwargs.pop('meta')
+    if "meta" in kwargs:
+        meta = kwargs.pop("meta")
     else:
         meta = _emulate(func, df, *args, **kwargs)
     meta = make_meta(meta, index=df._meta.index)
 
-    name = '{0}-{1}'.format(func_name, token)
-    name_a = 'overlap-prepend-' + tokenize(df, before)
-    name_b = 'overlap-append-' + tokenize(df, after)
+    name = "{0}-{1}".format(func_name, token)
+    name_a = "overlap-prepend-" + tokenize(df, before)
+    name_b = "overlap-append-" + tokenize(df, after)
     df_name = df._name
 
     dsk = {}
 
-    # Have to do the checks for too large windows in the time-delta case
-    # here instead of in `overlap_chunk`, since we can't rely on fix-frequency
-    # index
-
     timedelta_partition_message = (
         "Partition size is less than specified window. "
         "Try using ``df.repartition`` to increase the partition size"
     )
 
     if before and isinstance(before, Integral):
-        dsk.update({(name_a, i): (M.tail, (df_name, i), before)
-                    for i in range(df.npartitions - 1)})
+        dsk.update(
+            {
+                (name_a, i): (M.tail, (df_name, i), before)
+                for i in range(df.npartitions - 1)
+            }
+        )
         prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
     elif isinstance(before, datetime.timedelta):
         # Assumes monotonic (increasing?) index
-        deltas = pd.Series(df.divisions).diff().iloc[1:-1]
+        divs = pd.Series(df.divisions)
+        deltas = divs.diff().iloc[1:-1]
+
+        # In the first case window-size is larger than at least one partition, thus it is
+        # necessary to calculate how many partitions must be used for each rolling task.
+        # Otherwise, these calculations can be skipped (faster)
+
         if (before > deltas).any():
-            raise ValueError(timedelta_partition_message)
-        dsk.update({(name_a, i): (_tail_timedelta, (df_name, i), (df_name, i + 1), before)
-                    for i in range(df.npartitions - 1)})
-        prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
+            pt_z = divs[0]
+            for i in range(df.npartitions - 1):
+                # Select all indexes of relevant partitions between the current partition and
+                # the partition with the highest division outside the rolling window (before)
+                pt_i = divs[i + 1]
+
+                # lower-bound the search to the first division
+                lb = max(pt_i - before, pt_z)
+
+                first, j = divs[i], i
+                while first > lb and j > 0:
+                    first = first - deltas[j]
+                    j = j - 1
+
+                dsk.update(
+                    {
+                        (name_a, i): (
+                            _tail_timedelta,
+                            [(df_name, k) for k in range(j, i + 1)],
+                            (df_name, i + 1),
+                            before,
+                        )
+                    }
+                )
+
+            prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
+
+        else:
+            dsk.update(
+                {
+                    (name_a, i): (
+                        _tail_timedelta,
+                        [(df_name, i)],
+                        (df_name, i + 1),
+                        before,
+                    )
+                    for i in range(df.npartitions - 1)
+                }
+            )
+            prevs = [None] + [(name_a, i) for i in range(df.npartitions - 1)]
     else:
         prevs = [None] * df.npartitions
 
     if after and isinstance(after, Integral):
-        dsk.update({(name_b, i): (M.head, (df_name, i), after)
-                    for i in range(1, df.npartitions)})
+        dsk.update(
+            {
+                (name_b, i): (M.head, (df_name, i), after)
+                for i in range(1, df.npartitions)
+            }
+        )
         nexts = [(name_b, i) for i in range(1, df.npartitions)] + [None]
     elif isinstance(after, datetime.timedelta):
         # TODO: Do we have a use-case for this? Pandas doesn't allow negative rolling windows
@@ -129,15 +183,28 @@ def map_overlap(func, df, before, after, *args, **kwargs):
         if (after > deltas).any():
             raise ValueError(timedelta_partition_message)
 
-        dsk.update({(name_b, i): (_head_timedelta, (df_name, i - 0), (df_name, i), after)
-                    for i in range(1, df.npartitions)})
+        dsk.update(
+            {
+                (name_b, i): (_head_timedelta, (df_name, i - 0), (df_name, i), after)
+                for i in range(1, df.npartitions)
+            }
+        )
         nexts = [(name_b, i) for i in range(1, df.npartitions)] + [None]
     else:
         nexts = [None] * df.npartitions
 
     for i, (prev, current, next) in enumerate(zip(prevs, df.__dask_keys__(), nexts)):
-        dsk[(name, i)] = (overlap_chunk, func, prev, current, next, before,
-                          after, args, kwargs)
+        dsk[(name, i)] = (
+            overlap_chunk,
+            func,
+            prev,
+            current,
+            next,
+            before,
+            after,
+            args,
+            kwargs,
+        )
 
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[df])
     return df._constructor(graph, name, meta, df.divisions)
@@ -160,21 +227,24 @@ def _head_timedelta(current, next_, after):
     return next_[next_.index < (current.index.max() + after)]
 
 
-def _tail_timedelta(prev, current, before):
-    """Return rows of ``prev`` whose index is after the first
-    observation in ``current`` - ``before``.
+def _tail_timedelta(prevs, current, before):
+    """Return the concatenated rows of each dataframe in ``prevs`` whose
+    index is after the first observation in ``current`` - ``before``.
 
     Parameters
     ----------
     current : DataFrame
-    next_ : DataFrame
+    prevs : list of DataFrame objects
     before : timedelta
 
     Returns
     -------
     overlapped : DataFrame
     """
-    return prev[prev.index > (current.index.min() - before)]
+    selected = pd.concat(
+        [prev[prev.index > (current.index.min() - before)] for prev in prevs]
+    )
+    return selected
 
 
 def pandas_rolling_method(df, rolling_kwargs, name, *args, **kwargs):
@@ -185,13 +255,21 @@ def pandas_rolling_method(df, rolling_kwargs, name, *args, **kwargs):
 class Rolling(object):
     """Provides rolling window calculations."""
 
-    def __init__(self, obj, window=None, min_periods=None, freq=None,
-                 center=False, win_type=None, axis=0):
+    def __init__(
+        self,
+        obj,
+        window=None,
+        min_periods=None,
+        freq=None,
+        center=False,
+        win_type=None,
+        axis=0,
+    ):
         if freq is not None:
-            msg = 'The deprecated freq argument is not supported.'
+            msg = "The deprecated freq argument is not supported."
             raise NotImplementedError(msg)
 
-        self.obj = obj     # dataframe or series
+        self.obj = obj  # dataframe or series
         self.window = window
         self.min_periods = min_periods
         self.center = center
@@ -209,11 +287,13 @@ class Rolling(object):
         self._min_periods = pd_roll.min_periods
 
     def _rolling_kwargs(self):
-        return {'window': self.window,
-                'min_periods': self.min_periods,
-                'center': self.center,
-                'win_type': self.win_type,
-                'axis': self.axis}
+        return {
+            "window": self.window,
+            "min_periods": self.min_periods,
+            "center": self.center,
+            "win_type": self.win_type,
+            "axis": self.axis,
+        }
 
     @property
     def _has_single_partition(self):
@@ -221,114 +301,138 @@ class Rolling(object):
         Indicator for whether the object has a single partition (True)
         or multiple (False).
         """
-        return (self.axis in (1, 'columns') or
-                (isinstance(self.window, Integral) and self.window <= 1) or
-                self.obj.npartitions == 1)
+        return (
+            self.axis in (1, "columns")
+            or (isinstance(self.window, Integral) and self.window <= 1)
+            or self.obj.npartitions == 1
+        )
 
     def _call_method(self, method_name, *args, **kwargs):
         rolling_kwargs = self._rolling_kwargs()
-        meta = pandas_rolling_method(self.obj._meta_nonempty, rolling_kwargs,
-                                     method_name, *args, **kwargs)
+        meta = pandas_rolling_method(
+            self.obj._meta_nonempty, rolling_kwargs, method_name, *args, **kwargs
+        )
 
         if self._has_single_partition:
             # There's no overlap just use map_partitions
-            return self.obj.map_partitions(pandas_rolling_method,
-                                           rolling_kwargs, method_name,
-                                           *args, token=method_name, meta=meta,
-                                           **kwargs)
+            return self.obj.map_partitions(
+                pandas_rolling_method,
+                rolling_kwargs,
+                method_name,
+                *args,
+                token=method_name,
+                meta=meta,
+                **kwargs
+            )
         # Convert window to overlap
         if self.center:
             before = self.window // 2
             after = self.window - before - 1
-        elif self._win_type == 'freq':
+        elif self._win_type == "freq":
             before = pd.Timedelta(self.window)
             after = 0
         else:
             before = self.window - 1
             after = 0
-        return map_overlap(pandas_rolling_method, self.obj, before, after,
-                           rolling_kwargs, method_name, *args,
-                           token=method_name, meta=meta, **kwargs)
+        return map_overlap(
+            pandas_rolling_method,
+            self.obj,
+            before,
+            after,
+            rolling_kwargs,
+            method_name,
+            *args,
+            token=method_name,
+            meta=meta,
+            **kwargs
+        )
 
     @derived_from(pd_Rolling)
     def count(self):
-        return self._call_method('count')
+        return self._call_method("count")
 
     @derived_from(pd_Rolling)
     def sum(self):
-        return self._call_method('sum')
+        return self._call_method("sum")
 
     @derived_from(pd_Rolling)
     def mean(self):
-        return self._call_method('mean')
+        return self._call_method("mean")
 
     @derived_from(pd_Rolling)
     def median(self):
-        return self._call_method('median')
+        return self._call_method("median")
 
     @derived_from(pd_Rolling)
     def min(self):
-        return self._call_method('min')
+        return self._call_method("min")
 
     @derived_from(pd_Rolling)
     def max(self):
-        return self._call_method('max')
+        return self._call_method("max")
 
     @derived_from(pd_Rolling)
     def std(self, ddof=1):
-        return self._call_method('std', ddof=1)
+        return self._call_method("std", ddof=1)
 
     @derived_from(pd_Rolling)
     def var(self, ddof=1):
-        return self._call_method('var', ddof=1)
+        return self._call_method("var", ddof=1)
 
     @derived_from(pd_Rolling)
     def skew(self):
-        return self._call_method('skew')
+        return self._call_method("skew")
 
     @derived_from(pd_Rolling)
     def kurt(self):
-        return self._call_method('kurt')
+        return self._call_method("kurt")
 
     @derived_from(pd_Rolling)
     def quantile(self, quantile):
-        return self._call_method('quantile', quantile)
+        return self._call_method("quantile", quantile)
 
     @derived_from(pd_Rolling)
     def apply(self, func, args=(), kwargs={}, **kwds):
         # TODO: In a future version of pandas this will change to
         # raw=False. Think about inspecting the function signature and setting
         # to that?
-        if PANDAS_VERSION >= '0.23.0':
+        if PANDAS_VERSION >= "0.23.0":
             kwds.setdefault("raw", None)
         else:
             if kwargs:
-                msg = ("Invalid argument to 'apply'. Keyword arguments "
-                       "should be given as a dict to the 'kwargs' arugment. ")
+                msg = (
+                    "Invalid argument to 'apply'. Keyword arguments "
+                    "should be given as a dict to the 'kwargs' arugment. "
+                )
                 raise TypeError(msg)
-        return self._call_method('apply', func, args=args,
-                                 kwargs=kwargs, **kwds)
+        return self._call_method("apply", func, args=args, kwargs=kwargs, **kwds)
 
     @derived_from(pd_Rolling)
     def aggregate(self, func, args=(), kwargs={}, **kwds):
-        return self._call_method('agg', func, args=args,
-                                 kwargs=kwargs, **kwds)
+        return self._call_method("agg", func, args=args, kwargs=kwargs, **kwds)
 
     agg = aggregate
 
     def __repr__(self):
-
         def order(item):
             k, v = item
-            _order = {'window': 0, 'min_periods': 1, 'center': 2,
-                      'win_type': 3, 'axis': 4}
+            _order = {
+                "window": 0,
+                "min_periods": 1,
+                "center": 2,
+                "win_type": 3,
+                "axis": 4,
+            }
             return _order[k]
 
         rolling_kwargs = self._rolling_kwargs()
         # pandas translates the '2S' offset to nanoseconds
-        rolling_kwargs['window'] = self._window
-        rolling_kwargs['win_type'] = self._win_type
-        return 'Rolling [{}]'.format(','.join(
-            '{}={}'.format(k, v)
-            for k, v in sorted(rolling_kwargs.items(), key=order)
-            if v is not None))
+        rolling_kwargs["window"] = self._window
+        rolling_kwargs["win_type"] = self._win_type
+        return "Rolling [{}]".format(
+            ",".join(
+                "{}={}".format(k, v)
+                for k, v in sorted(rolling_kwargs.items(), key=order)
+                if v is not None
+            )
+        )
